unit Main;

{ Contents copyright (c) 1995 by Charles Calvert }
{ Project Name: DELALLDB

{ This program allows you to selectively delete
  files from your hard drive. It deletes files
  by extension, so you can delete all the files
  with a BAK extension, etc. It cannot be used
  to delete files that have no extension.

  It will also count the size of directories
  on your system, and add up the total sizes.

  It saves the DirSizeList in a DB file which you
  can sort and perform calculations on. Other
  information is stored in text files that you
  can reach through the File menu for the program.

  A special file saves the list of extensions of
  the types of files you want to delete. This list
  can be edited at run time, and it is displayed
  in a list box.

  There are special cases for files that end with
  an EXE or ~?? extensions. To delete EXE files, '*.EXE'
  must be in the EXTLIST.TXT file, and also the
  Delete EXEs checkbox must be marked. To delete files
  whose extension begins with a tilda, just check
  the appropriate box. There is no need to put this
  extension in the EXTLIST file. }

interface

uses
  SysUtils, WinTypes, WinProcs,
  Messages, Classes, Graphics,
  Controls, Forms, Dialogs,
  StdCtrls, AllDirs, Fileiter,
  Menus, Buttons, ExtCtrls,
  FileCtrl, DB, DBTables;

type
  TForm1 = class(TForm)
    FileIterator1: TFileIterator;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Open1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    DeleteBtn: TBitBtn;
    Panel1: TPanel;
    ExtBox: TListBox;
    BitBtn2: TBitBtn;
    DirectoryListBox1: TDirectoryListBox;
    DriveComboBox1: TDriveComboBox;
    OpenDelFile1: TMenuItem;
    Panel2: TPanel;
    DelExe: TCheckBox;
    DelTilda: TCheckBox;
    OpenDirSizeFile1: TMenuItem;
    Table1: TTable;
    Table1Directory: TStringField;
    Table1Size: TFloatField;
    pDirShow: TPanel;
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FileIterator1FoundFile(FileName: String; SR: TSearchRec);
    procedure FileIterator1ProcessDir(DirName: String);
  private
    FDelTilda: Boolean;
    FDelExes: Boolean;
    FDirSize: LongInt;
    FDeleteList: TStringList;
    FDirSizeList: TStringList;
    FStartPath: string;
    FDirName: string;
    FDirSizeName: string;
    FDelListName: string;
    FExtBoxName: string;
    procedure SaveDirList;
    procedure SetUpLists;
    function LoadExtBox: Boolean;
  end;

var
  Form1: TForm1;

implementation

uses
  Grider,
  StrBox,
  UtilBox;

{$R *.DFM}


procedure TForm1.SaveDirList;
var
  ListStr: string;
  S1, S2: string;
  i: Integer;
begin
  Cursor := crHourGlass;
  try
    Table1.Close;
    Table1.EmptyTable;
    Table1.Open;
    for i := 0 to FDirSizeList.Count - 1 do begin
      Table1.Insert;
      ListStr := FDirSizeList.Strings[i];
      S1 := GetFirstToken(ListStr, '=');
      S2 := GetLastToken(ListStr, '=');
      Table1.FieldByName('Directory').AsString := S1;
      Table1.FieldByName('Size').AsString := S2;
      Table1.Post;
    end;
  finally
    Cursor := crDefault;
  end;
end;

function TForm1.LoadExtBox: Boolean;
begin
  ExtBox.Items.LoadFromFile(FExtBoxName);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  ExtBox.Items.SaveToFile(FExtBoxName);
  FDeleteList.Free;
  FDirSizeList.Free;
end;

procedure TForm1.SetUpLists;
begin
  FDeleteList.Free;
  FDirSizeList.Free;
  FDeleteList := TStringList.Create;
  FDirSizeList := TStringList.Create;
  FDirName := '';
  FDirSize := 0;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FStartPath := StripLastToken(ParamStr(0), '\');
  FDirSizeName := FStartPath + '\dirsize.txt';
  FDelListName := FStartPath + '\dellist.txt';
  FExtBoxName := FStartPath + '\extbox.txt';
  SetUpLists;
  LoadExtBox;
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Open1Click(Sender: TObject);
var
  SPtr: array[0..150] of char;
  S: string;
begin
  case TMenuItem(Sender).Tag of
    100: S := FExtBoxName;
    101: S := FDelListName;
    102: begin
      GridView.DataSource1.DataSet := Table1;
      Table1.Open;
      GridView.ShowModal;
      Table1.Close;
      Exit;
    end;
  end;
  StrPCopy(SPtr, 'notepad.exe ' + S);
  WinExecAndWait(SPtr, sw_ShowNormal);
  LoadExtBox;
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
begin
  Close;
end;

{ Note that if you want to change the cursor for
  an entire application, then change Screen.Cursor. }
procedure TForm1.DeleteBtnClick(Sender: TObject);
var
  i: Integer;
  S: string;
begin
  FDelTilda := DelTilda.Checked;
  FDelExes := DelExe.Checked;
  s := DirectoryListBox1.Directory;
  if S[Length(S)] <> '\' then S := S + '\';
  if MessageDlg('Delete from: ' + S, mtConfirmation,
                  [mbYes, mbNo], 0) = idNo then Exit;
  try
    Screen.Cursor := crHourGlass;
    SetUpLists;
    FileIterator1.Run('*.*', DirectoryListBox1.Directory);
    FDeleteList.SaveToFile(FDelListName);
    FDirSizeList.Add(FDirName + ' = ' + IntToStr(FDirSize));
    DeleteFile(FDirSizeName);
    SaveDirList;
    FDirSizeList.SaveToFile(FDirSizeName);
    for i := 0 to FDeleteList.Count - 1 do
      DeleteFile(FDeleteList.Strings[i]);
  finally
    Screen.Cursor := crDefault;
    pDirShow.Caption := '';
  end;
end;

{ This is the place where the list of files to delete is
  created. Get the extension, see if it is in the delete
  list, if it is, add it to the delete list.

  There are special cases for handling EXEs. Specifically, you
  want to add the files to the TotalDirSize always but only
  delete them if they are listed in the EXELIST.TXT file and
  Delete EXEs is checked. Also a special case for the files
  whose extension begin with a tilda. Finally, a special case
  for the possibility that the user put a carraige return
  at the end of the EXELIST thereby accidentally requesting
  that all files with NO extension get deleted. In other
  words, this program cannot be used to delete all files
  that have NO extension. }

procedure TForm1.FileIterator1FoundFile(FileName: String; SR: TSearchRec);
var
  S: string;
  i: Integer;
  AddToDirSize: Boolean;
begin
  AddToDirSize := False;
  S := UpperCase(ExtractFileExt(FileName));
  for i := 0 to ExtBox.Items.Count - 1 do begin
    if (S = '.EXE') and (not FDelExes) then begin
      AddToDirSize := True;
      Continue;
    end;
    if (S = UpperCase(ExtBox.Items.Strings[i])) and (StripBlanks(S) <> '') then begin
      FDeleteList.Add(FileName);
      Exit;
    end;
    if not FDelTilda then
      AddToDirSize := True
    else if S[2] = '~' then begin
      FDeleteList.Add(FileName);
      Exit;
    end;
    AddToDirSize := True;
  end;
  if AddToDirSize then FDirSize := FDirSize + SR.Size;
end;

procedure TForm1.FileIterator1ProcessDir(DirName: String);
begin
  pDirShow.Caption := DirName;
  Application.ProcessMessages;
  if FDirName <> '' then
    FDirSizeList.Add(FDirName + ' = ' + IntToStr(FDirSize));
  FDirName := DirName;
  FDirSize := 0;
end;

end.
